package java.time.format;

import java.time.ZoneId;
import java.util.Objects;
import java.time.temporal.TemporalAccessor;
import java.time.temporal.TemporalField;
import java.util.Set;
import java.time.chrono.IsoChronology;
import java.util.Locale;
import java.time.chrono.Chronology;
import java.util.function.Consumer;
import java.util.ArrayList;

final class DateTimeParseContext
{
    private DateTimeFormatter formatter;
    private boolean caseSensitive;
    private boolean strict;
    private final ArrayList<Parsed> parsed;
    private ArrayList<Consumer<Chronology>> chronoListeners;
    
    DateTimeParseContext(DateTimeFormatter formatter) {
        this.caseSensitive = true;
        this.strict = true;
        this.parsed = new ArrayList();
        this.chronoListeners = null;
        this.formatter = formatter;
        this.parsed.add(new Parsed());
    }
    
    DateTimeParseContext copy() {
        DateTimeParseContext dateTimeParseContext = new DateTimeParseContext(this.formatter);
        dateTimeParseContext.caseSensitive = this.caseSensitive;
        dateTimeParseContext.strict = this.strict;
        return dateTimeParseContext;
    }
    
    Locale getLocale() {
        return this.formatter.getLocale();
    }
    
    DecimalStyle getDecimalStyle() {
        return this.formatter.getDecimalStyle();
    }
    
    Chronology getEffectiveChronology() {
        Chronology chronology = this.currentParsed().chrono;
        if (chronology == null) {
            chronology = this.formatter.getChronology();
            if (chronology == null) {
                chronology = IsoChronology.INSTANCE;
            }
        }
        return chronology;
    }
    
    boolean isCaseSensitive() {
        return this.caseSensitive;
    }
    
    void setCaseSensitive(boolean caseSensitive) {
        this.caseSensitive = caseSensitive;
    }
    
    boolean subSequenceEquals(CharSequence charSequence, int n, CharSequence charSequence2, int n2, int n3) {
        if (n + n3 > charSequence.length() || n2 + n3 > charSequence2.length()) {
            return false;
        }
        if (this.isCaseSensitive()) {
            int j = 0;
            while (j < n3) {
                if (charSequence.charAt(n + j) != charSequence2.charAt(n2 + j)) {
                    return false;
                }
                ++j;
            }
        }
        else {
            int i = 0;
            while (i < n3) {
                int char1 = charSequence.charAt(n + i);
                int char2 = charSequence2.charAt(n2 + i);
                if (char1 != char2 && Character.toUpperCase((char)char1) != Character.toUpperCase((char)char2) && Character.toLowerCase((char)char1) != Character.toLowerCase((char)char2)) {
                    return false;
                }
                ++i;
            }
        }
        return true;
    }
    
    boolean charEquals(char c, char c2) {
        if (this.isCaseSensitive()) {
            return c == c2;
        }
        return charEqualsIgnoreCase(c, c2);
    }
    
    static boolean charEqualsIgnoreCase(char c, char c2) {
        return c == c2 || Character.toUpperCase(c) == Character.toUpperCase(c2) || Character.toLowerCase(c) == Character.toLowerCase(c2);
    }
    
    boolean isStrict() {
        return this.strict;
    }
    
    void setStrict(boolean strict) {
        this.strict = strict;
    }
    
    void startOptional() {
        this.parsed.add(this.currentParsed().copy());
    }
    
    void endOptional(boolean b) {
        if (b) {
            this.parsed.remove(this.parsed.size() - 2);
        }
        else {
            this.parsed.remove(this.parsed.size() - 1);
        }
    }
    
    private Parsed currentParsed() {
        return (Parsed)this.parsed.get(this.parsed.size() - 1);
    }
    
    Parsed toUnresolved() {
        return this.currentParsed();
    }
    
    TemporalAccessor toResolved(ResolverStyle resolverStyle, Set<TemporalField> set) {
        Parsed currentParsed = this.currentParsed();
        currentParsed.chrono = this.getEffectiveChronology();
        currentParsed.zone = ((currentParsed.zone != null) ? currentParsed.zone : this.formatter.getZone());
        return currentParsed.resolve(resolverStyle, set);
    }
    
    Long getParsed(TemporalField temporalField) {
        return (Long)this.currentParsed().fieldValues.get(temporalField);
    }
    
    int setParsedField(TemporalField obj, long l, int n, int n2) {
        Objects.requireNonNull(obj, "field");
        Long n3 = (Long)this.currentParsed().fieldValues.put(obj, Long.valueOf(l));
        return (n3 != null && (long)n3 != l) ? (~n) : n2;
    }
    
    void setParsed(Chronology chronology) {
        Objects.requireNonNull(chronology, "chrono");
        this.currentParsed().chrono = chronology;
        if (this.chronoListeners != null && !this.chronoListeners.isEmpty()) {
            Consumer[] array = (Consumer[])this.chronoListeners.toArray(new Consumer[1]);
            this.chronoListeners.clear();
            for (int length = array.length, i = 0; i < length; ++i) {
                array[i].accept(chronology);
            }
        }
    }
    
    void addChronoChangedListener(Consumer<Chronology> e) {
        if (this.chronoListeners == null) {
            this.chronoListeners = new ArrayList();
        }
        this.chronoListeners.add(e);
    }
    
    void setParsed(ZoneId zoneId) {
        Objects.requireNonNull(zoneId, "zone");
        this.currentParsed().zone = zoneId;
    }
    
    void setParsedLeapSecond() {
        this.currentParsed().leapSecond = true;
    }
    
    @Override
    public String toString() {
        return this.currentParsed().toString();
    }
}
